
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/md.h>

char **parser (char*, int);

static int open_or_die (char *file)
{
  int fd;
  
  if ((fd=open (file, O_RDONLY))==-1)
  {
    perror (file);
    exit (EXIT_FAILURE);
  }

  return fd;
}

static void do_mdadd (int fd, char *dev)
{
  struct stat s;

  if (stat (dev, &s))
  {
    perror (dev);
    return;
  }

  if (MAJOR (s.st_rdev)==FLOPPY_MAJOR)
    fprintf (stderr, "%s : warning, md cannot use floppy\n", dev);
  
  if (ioctl (fd, REGISTER_DEV, s.st_rdev))
    perror (dev);
}

static void do_mdstart (int fd, char *dev)
{
  if (ioctl (fd, START_MD, 0))
    perror (dev);
}

static void do_mdstop (int fd, char *dev)
{
  if (ioctl (fd, STOP_MD, 0))
    perror (dev);
}

int main (int argc, char *argv[])
{
  int i, j, fd, all=0, run=0, flag;
  char *namestart=argv[0];
  char **ent;

  while (1)
  {
    if ((flag=getopt (argc, argv, "ar"))==-1)
      break;

    switch (flag)
    {
      case 'a':
        all=1;
        break;
      
      case 'r':
	run=1;
	break;

      default:
	exit (EXIT_FAILURE);
    }
  }
         
  if (!all && optind==argc)
  {
    fprintf (stderr, "usage :\t%s [-r] md-device ...\n", argv[0]);
    fprintf (stderr, "usage :\t%s -a [-r]\n", argv[0]);
    exit (EXIT_FAILURE);
  }

  if (!all)
    fd=open_or_die (argv[optind]);
  else
    fd=-1;			/* Should never happend ! */

  for (i=0; i<strlen (argv[0]); i++)
    if (argv[0][i]=='/')
      namestart=argv[0]+i+1;
  
  if (!strcmp (namestart, "mdadd"))
  {
    if (all)
    {
      i=1;
      while ((ent=parser (NULL, i++)))
      {
	fd=open_or_die (ent[0]);

	j=1;
	while (ent[j])
	{
	  do_mdadd (fd, ent[j]);
	  free (ent[j]);
	  j++;
	}

	if (run)
	  do_mdstart (fd, ent[0]);
	free (ent[0]);
      }
    }
    else
    {
      if ((argc-optind)==1)
	if ((ent=parser (argv[optind], 0)))
	{
	  free (ent[0]);
	  
	  j=1;
	  while (ent[j])
	  {
	    do_mdadd (fd, ent[j]);
	    free (ent[j]);
	    j++;
	  }
	}
	else
	{
	  fprintf (stderr, "Unknown device %s\n", argv[optind]);
	  exit (EXIT_FAILURE);
	}
      else
	for (i=optind+1; i<argc; i++)
	  do_mdadd (fd, argv[i]);
      
      if (run)
	do_mdstart (fd, argv[optind]);
    }
    return (0);
  }
  
  if (!strcmp (namestart, "mdrun"))
  {
    if (all)
    {
      i=1;
      while ((ent=parser (NULL, i++)))
      {
	fd=open_or_die (ent[0]);
	do_mdstart (fd, ent[0]);
	free (ent[0]);

	j=1;
	while (ent[j])
	  free (ent[j++]);
      }
    }
    else
      do_mdstart (fd, argv[optind]);
	
    return (0);
  }

  if (!strcmp (namestart, "mdstop"))
  {
    if (all)
    {
      i=1;
      while ((ent=parser (NULL, i++)))
      {
	fd=open_or_die (ent[0]);
	do_mdstop (fd, ent[0]);
	free (ent[0]);

	j=1;
	while (ent[j])
	  free (ent[j++]);
      }
    }
    else
      do_mdstop (fd, argv[optind]);
	
    return (0);
  }

  fprintf (stderr, "Unknown command %s\n", argv[0]);
  return (EXIT_FAILURE);
}

